<?php

namespace Drupal\iconset\Plugin\Iconset;

use Drupal\Component\Plugin\Definition\PluginDefinitionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Extension\ExtensionPathResolver;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\ImageToolkit\ImageToolkitInterface;
use Drupal\Core\ImageToolkit\ImageToolkitManager;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\iconset\IconsetInterface;
use Drupal\iconset\Asset\AssetInterface;
use Drupal\iconset\Asset\ImageAsset;
use Drupal\iconset\Attribute\IconsetIconHandler;
use Drupal\iconset\Plugin\IconHandlerFileBase;

/**
 * Icon plugin for managing SVG icons and sprite files.
 *
 * Icon plugin handler for reading and determining the contents of an SVG file.
 * SVG files can just be an icon or contain a collection of symbols (sprites).
 *
 * A file with sprites either has "symbol" elements, which each represent icons,
 * or can be "g" elements (groups). Group elements are only considered an icon
 * if they are found in a "defs" element.
 *
 * It is recommend to use SVG sprite files, and to use "symbol" elements for
 * each icon in the sprite. Not only is it more efficient, the icons are also
 * easier to theme, because you can apply CSS properties like fill to them.
 */
#[IconsetIconHandler(
  id: 'image',
  label: new TranslatableMarkup('SVG icons and sprites'),
  help: new TranslatableMarkup('Enumerate and render icons from SVG files.'),
  allow_directory: TRUE,
  libraries: [
    'iconset/svg-icons',
  ],
)]
class ImgIconHandler extends IconHandlerFileBase {

  /**
   * Image toolkit instance for image manipulation.
   *
   * @var \Drupal\Core\ImageToolkit\ImageToolkitInterface
   */
  protected ImageToolkitInterface $imageToolkit;

  /**
   * Create a new instance of the image icon plugin handler.
   *
   * @param array $configuration
   *   Plugin configuration.
   * @param string $plugin_id
   *   The plugin ID.
   * @param array|\Drupal\Component\Plugin\Definition\PluginDefinitionInterface $plugin_definition
   *   The plugin definition.
   * @param \Drupal\Core\File\FileSystemInterface $filesystem
   *   Filesystem service.
   * @param \Drupal\Core\Extension\ExtensionPathResolver $extensionPathResolver
   *   The extension path resolver.
   * @param \Drupal\Core\ImageToolkit\ImageToolkitManager $imgToolkitManager
   *   The image toolkit manager to getting an image toolkit instance.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    array|PluginDefinitionInterface $plugin_definition,
    FileSystemInterface $filesystem,
    ExtensionPathResolver $extensionPathResolver,
    ImageToolkitManager $imgToolkitManager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $filesystem, $extensionPathResolver);

    $this->imageToolkit = $imgToolkitManager->getDefaultToolkit();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('file_system'),
      $container->get('extension.path.resolver'),
      $container->get('image.toolkit.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFileExtensions(): array {
    return $this->imageToolkit->getSupportedExtensions();
  }

  /**
   * {@inheritdoc}
   */
  public function createFileAssets(string $filepath, string|array $asset_info, IconsetInterface $iconset): array|AssetInterface|null {
    try {
      $name = $this->filesystem->basename($filepath);
      $name = preg_replace('/\..+$/', '', $name);

      $this->imageToolkit->setSource($filepath);
      $this->imageToolkit->parseFile();

      $icon = [];
      $icon['id'] = Html::cleanCssIdentifier($name);
      $icon['label'] = $name;
      $icon['width'] = $this->imageToolkit->getWidth();
      $icon['height'] = $this->imageToolkit->getHeight();

      return new ImageAsset($filepath, $icon);
    }
    catch (\Exception $e) {
      $this
        ->getLogger('iconset')
        ->error('Unable to read image file at @line of @file: @message', [
          '@line' => $e->getLine(),
          '@file' => $e->getFile(),
          '@message' => $e->getMessage(),
        ]);
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getJsSettings(): array {
    return [
      'type' => 'image',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function formatJson(AssetInterface $asset): array {
    /** @var \Drupal\iconset\Asset\FileAssetInterface $asset */
    $icons = $asset->getIcons();
    $icon = reset($icons);

    return $icon + [
      'file' => base_path() . $asset->getFilepath(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function build(string $icon_id, AssetInterface $asset, array $options = []): array {
    /** @var \Drupal\iconset\Asset\FileAssetInterface $asset */
    $icon = $asset->getIcon($icon_id);
    $url = Url::fromUri('base:' . $asset->getFilepath());

    $attrs = [];
    $attrs['class'] = ['icon', 'icon--img'];

    if (empty($options['decorative'])) {
      $attrs['alt'] = $options['alt'] ?? $icon['label'];
    }
    else {
      $attrs['role'] = 'presentation';
      $attrs['alt'] = '';
    }

    // To ensure the least amount of resizing of the HTML element output,
    // apply the image dimensions as best can be calculated. If both dimensions
    // are provided, use them directly, otherwise scale the image appropriately.
    if (!empty($options['height'])) {
      $attrs['height'] = intval($options['height']);

      // Scale the width based on the height ratio scaling if no explicit width.
      $attrs['width'] = empty($options['width'])
        ? $icon['width'] * $attrs['height'] / $icon['height']
        : intval($options['width']);
    }
    elseif (!empty($options['width'])) {
      $attrs['width'] = intval($options['width']);
      $attrs['height'] = $icon['height'] * $attrs['width'] / $icon['width'];
    }
    else {
      $attrs['width'] = $icon['width'];
      $attrs['height'] = $icon['height'];
    }

    return [
      '#theme' => 'image',
      '#uri' => $url->toString(),
      '#attributes' => $attrs,
    ];
  }

}
